@@ -66,13 +66,13 @@ def copy_as_neuroncriteria(obj, client=None):
             'If passing a DataFrame as NeuronCriteria, it must have "bodyId" column'
         return NeuronCriteria(bodyId=obj['bodyId'].values, client=client)
 
+    if obj is None:
+        return NeuronCriteria(client=client)
+
+    if isinstance(obj, NeuronCriteria):
+        return copy.copy(obj)
+
     if not isinstance(obj, Collection) or isinstance(obj, str):
-        if obj is None:
-            return NeuronCriteria(client=client)
-
-        if isinstance(obj, NeuronCriteria):
-            return copy.copy(obj)
-
         if isinstance(obj, str):
             return NeuronCriteria(type=obj, instance=obj, client=client)
 
@@ -126,13 +126,14 @@ class NeuronCriteria:
     """
 
     @inject_client
-    @make_args_iterable(['bodyId', 'instance', 'type', 'status', 'rois', 'inputRois', 'outputRois'])
+    @make_args_iterable(['bodyId', 'instance', 'type', 'cellBodyFiber', 'status', 'rois', 'inputRois', 'outputRois'])
     def __init__( self, matchvar='n', *,
                   bodyId=None, instance=None, type=None, regex=False,
+                  cellBodyFiber=None,
                   status=None, cropped=None,
                   min_pre=0, min_post=0,
                   rois=None, inputRois=None, outputRois=None, min_roi_inputs=1, min_roi_outputs=1,
-                  label='Neuron', roi_req='all',
+                  label=None, roi_req='all',
                   client=None ):
         """
         Except for ``matchvar``, all parameters must be passed as keyword arguments.
@@ -181,6 +182,10 @@ class NeuronCriteria:
                 If ``True``, the ``instance`` and ``type`` arguments will be interpreted as
                 regular expressions, rather than exact match strings.
 
+            cellBodyFiber (str or list of str):
+                Matches for the neuron ``cellBodyFiber`` field.  To search for neurons
+                with no CBF at all, use ``cellBodyFiber=[None]``.
+
             status (str or list of str):
                 Matches for the neuron ``status`` field.  To search for neurons with no status
                 at all, use ``status=[None]``.
@@ -224,6 +229,9 @@ class NeuronCriteria:
             label (Either ``'Neuron'`` or ``'Segment'``):
                 Which node label to match with.
                 (In neuprint, all ``Neuron`` nodes are also ``Segment`` nodes.)
+                By default, ``'Neuron'`` is used, unless you provided a non-empty ``bodyId`` list.
+                In that case, ``'Segment'`` is the default. (It's assumed you're really interested
+                in the bodies you explicitly listed, whether or not they have the ``'Neuron'`` label.)
 
             client (:py:class:`neuprint.client.Client`):
                 Used to validate ROI names.
@@ -242,20 +250,26 @@ class NeuronCriteria:
             (f"matchvar contains invalid characters: '{matchvar}'. "
              "Did you mean to pass this as a type or instance?")
 
-        assert label in ('Neuron', 'Segment'), f"Invalid label: {label}"
         assert len(bodyId) == 0 or np.issubdtype(np.asarray(bodyId).dtype, np.integer), \
             "bodyId should be an integer or list of integers"
 
+        if not label:
+            if len(bodyId) == 0:
+                label = 'Neuron'
+            else:
+                label = 'Segment'
+        assert label in ('Neuron', 'Segment'), f"Invalid label: {label}"
+
         assert not regex or len(instance) <= 1, \
             "Please provide only one regex pattern for instance"
         assert not regex or len(type) <= 1, \
             "Please provide only one regex pattern for type"
 
-        if not regex and isinstance(instance, str) and len(instance) == 1:
+        if not regex and instance and len(instance) == 1 and isinstance(instance[0], str):
             assert '.*' not in instance[0], \
                 f"instance appears to be a regular expression ('{instance[0]}'), but you didn't pass regex=True"
 
-        if not regex and isinstance(type, str) and len(type) == 1:
+        if not regex and type and len(type) == 1 and isinstance(type[0], str):
             assert '.*' not in type[0], \
                 f"type appears to be a regular expression ('{type[0]}'), but you didn't pass regex=True"
 
@@ -295,6 +309,7 @@ class NeuronCriteria:
         self.bodyId = bodyId
         self.instance = instance
         self.type = type
+        self.cellBodyFiber = cellBodyFiber
         self.status = status
         self.cropped = cropped
         self.min_pre = min_pre
@@ -367,6 +382,11 @@ class NeuronCriteria:
         if self.regex:
             s += ", regex=True"
 
+        if len(self.cellBodyFiber) == 1:
+            s += f', cellBodyFiber="{self.cellBodyFiber[0]}"'
+        elif len(self.instance) > 1:
+            s += f", cellBodyFiber={list(self.cellBodyFiber)}"
+
         if len(self.status) == 1:
             s += f', status="{self.status[0]}"'
         elif len(self.instance) > 1:
@@ -436,6 +456,11 @@ class NeuronCriteria:
             var = f"{self.matchvar}_search_statuses"
             exprs[var] = f"{[*values]} as {var}"
 
+        if len(self.cellBodyFiber) > self.MAX_LITERAL_LENGTH:
+            values = [*filter(lambda s: s is not None, self.cellBodyFiber)]
+            var = f"{self.matchvar}_search_CBFs"
+            exprs[var] = f"{[*values]} as {var}"
+
         return exprs
 
 
@@ -461,7 +486,7 @@ class NeuronCriteria:
         They're intended be combined (via 'AND') in
         the WHERE clause of a cypher query.
         """
-        exprs = [self.bodyId_expr(), self.typeinst_expr(), self.status_expr(),
+        exprs = [self.bodyId_expr(), self.typeinst_expr(), self.cbf_expr(), self.status_expr(),
                  self.cropped_expr(), self.rois_expr(), self.pre_expr(), self.post_expr()]
         exprs = [*filter(None, exprs)]
         return exprs
@@ -501,6 +526,12 @@ class NeuronCriteria:
             valuevar = f"{self.matchvar}_search_types"
         return where_expr('type', self.type, self.regex, self.matchvar, valuevar)
 
+    def cbf_expr(self):
+        valuevar = None
+        if len(self.cellBodyFiber) > self.MAX_LITERAL_LENGTH:
+            valuevar = f"{self.matchvar}_search_CBFs"
+        return where_expr('cellBodyFiber', self.cellBodyFiber, False, self.matchvar, valuevar)
+
     def status_expr(self):
         valuevar = None
         if len(self.status) > self.MAX_LITERAL_LENGTH:
@@ -809,6 +840,3 @@ def where_expr(field, values, regex=False, matchvar='n', valuevar=None):
             return f"{matchvar}.{field} in {valuevar} OR NOT exists({matchvar}.{field})"
         else:
             return f"{matchvar}.{field} in {[*values]} OR NOT exists({matchvar}.{field})"
-
-
-
